@@ -40,6 +40,8 @@ xmlDebugDumpString(FILE * output, const xmlChar * str)
 {
     int i;
 
+    if (output == NULL)
+	output = stdout;
     if (str == NULL) {
         fprintf(output, "(NULL)");
         return;
@@ -67,6 +69,11 @@ xmlDebugDumpDtdNode(FILE *output, xmlDtdPtr dtd, int depth) {
 
     fprintf(output, shift);
 
+    if (dtd == NULL) {
+	fprintf(output, "DTD node is NULL\n");
+	return;
+    }
+
     if (dtd->type != XML_DTD_NODE) {
 	fprintf(output, "PBM: not a DTD\n");
 	return;
@@ -116,6 +123,10 @@ xmlDebugDumpAttrDecl(FILE *output, xmlAttributePtr attr, int depth) {
 
     fprintf(output, shift);
 
+    if (attr == NULL) {
+	fprintf(output, "Attribute declaration is NULL\n");
+	return;
+    }
     if (attr->type != XML_ATTRIBUTE_DECL) {
 	fprintf(output, "PBM: not a Attr\n");
 	return;
@@ -233,6 +244,10 @@ xmlDebugDumpElemDecl(FILE *output, xmlElementPtr elem, int depth) {
 
     fprintf(output, shift);
 
+    if (elem == NULL) {
+	fprintf(output, "Element declaration is NULL\n");
+	return;
+    }
     if (elem->type != XML_ELEMENT_DECL) {
 	fprintf(output, "PBM: not a Elem\n");
 	return;
@@ -307,6 +322,10 @@ xmlDebugDumpEntityDecl(FILE *output, xmlEntityPtr ent, int depth) {
 
     fprintf(output, shift);
 
+    if (ent == NULL) {
+	fprintf(output, "Entity declaration is NULL\n");
+	return;
+    }
     if (ent->type != XML_ENTITY_DECL) {
 	fprintf(output, "PBM: not a Entity decl\n");
 	return;
@@ -391,6 +410,11 @@ xmlDebugDumpNamespace(FILE *output, xmlNsPtr ns, int depth) {
     shift[2 * i] = shift[2 * i + 1] = 0;
 
     fprintf(output, shift);
+
+    if (ns == NULL) {
+	fprintf(output, "namespace node is NULL\n");
+	return;
+    }
     if (ns->type != XML_NAMESPACE_DECL) {
         fprintf(output, "invalid namespace node %d\n", ns->type);
 	return;
@@ -429,6 +453,11 @@ xmlDebugDumpEntity(FILE *output, xmlEntityPtr ent, int depth) {
     shift[2 * i] = shift[2 * i + 1] = 0;
 
     fprintf(output, shift);
+    
+    if (ent == NULL) {
+	fprintf(output, "Entity is NULL\n");
+	return;
+    }
     switch (ent->etype) {
         case XML_INTERNAL_GENERAL_ENTITY:
 	    fprintf(output, "INTERNAL_GENERAL_ENTITY ");
@@ -487,7 +516,11 @@ xmlDebugDumpAttr(FILE *output, xmlAttrPtr attr, int depth) {
     shift[2 * i] = shift[2 * i + 1] = 0;
 
     fprintf(output, shift);
-
+    
+    if (attr == NULL) {
+	fprintf(output, "Attr is NULL");
+	return;
+    }
     fprintf(output, "ATTRIBUTE ");
     xmlDebugDumpString(output, attr->name);
     fprintf(output, "\n");
@@ -527,6 +560,8 @@ xmlDebugDumpAttr(FILE *output, xmlAttrPtr attr, int depth) {
 void
 xmlDebugDumpAttrList(FILE * output, xmlAttrPtr attr, int depth)
 {
+    if (output == NULL)
+	output = stdout;
     while (attr != NULL) {
         xmlDebugDumpAttr(output, attr, depth);
         attr = attr->next;
@@ -547,10 +582,17 @@ xmlDebugDumpOneNode(FILE * output, xmlNodePtr node, int depth)
     int i;
     char shift[100];
 
+    if (output == NULL)
+	output = stdout;
     for (i = 0; ((i < depth) && (i < 25)); i++)
         shift[2 * i] = shift[2 * i + 1] = ' ';
     shift[2 * i] = shift[2 * i + 1] = 0;
 
+    if (node == NULL) {
+	fprintf(output, shift);
+	fprintf(output, "node is NULL\n");
+	return;
+    }
     switch (node->type) {
         case XML_ELEMENT_NODE:
             fprintf(output, shift);
@@ -704,6 +746,20 @@ xmlDebugDumpOneNode(FILE * output, xmlNodePtr node, int depth)
 void
 xmlDebugDumpNode(FILE * output, xmlNodePtr node, int depth)
 {
+    if (output == NULL)
+	output = stdout;
+    if (node == NULL) {
+	int i;
+	char shift[100];
+    	
+	for (i = 0; ((i < depth) && (i < 25)); i++)
+	    shift[2 * i] = shift[2 * i + 1] = ' ';
+	shift[2 * i] = shift[2 * i + 1] = 0;
+	
+	fprintf(output, shift);
+	fprintf(output, "node is NULL\n");
+	return;
+    }	
     xmlDebugDumpOneNode(output, node, depth);
     if ((node->children != NULL) && (node->type != XML_ENTITY_REF_NODE))
         xmlDebugDumpNodeList(output, node->children, depth + 1);
@@ -720,6 +776,8 @@ xmlDebugDumpNode(FILE * output, xmlNodePtr node, int depth)
 void
 xmlDebugDumpNodeList(FILE * output, xmlNodePtr node, int depth)
 {
+    if (output == NULL)
+	output = stdout;
     while (node != NULL) {
         xmlDebugDumpNode(output, node, depth);
         node = node->next;
@@ -845,8 +903,12 @@ xmlDebugDumpDocument(FILE * output, xmlDocPtr doc)
 void
 xmlDebugDumpDTD(FILE * output, xmlDtdPtr dtd)
 {
-    if (dtd == NULL)
+    if (output == NULL)
+	output = stdout;
+    if (dtd == NULL) {
+	fprintf(output, "DTD is NULL\n");
         return;
+    }
     if (dtd->type != XML_DTD_NODE) {
         fprintf(output, "PBM: not a DTD\n");
         return;
@@ -890,6 +952,10 @@ xmlDebugDumpDTD(FILE * output, xmlDtdPtr dtd)
 
 static void
 xmlDebugDumpEntityCallback(xmlEntityPtr cur, FILE *output) {
+    if (cur == NULL) {
+	fprintf(output, "Entity is NULL");
+	return;
+    }
     fprintf(output, "%s : ", cur->name);
     switch (cur->etype) {
 	case XML_INTERNAL_GENERAL_ENTITY:
@@ -1015,6 +1081,9 @@ int
 xmlLsCountNode(xmlNodePtr node) {
     int ret = 0;
     xmlNodePtr list = NULL;
+    
+    if (node == NULL)
+	return(0);
 
     switch (node->type) {
 	case XML_ELEMENT_NODE:
@@ -1071,6 +1140,10 @@ xmlLsCountNode(xmlNodePtr node) {
  */
 void
 xmlLsOneNode(FILE *output, xmlNodePtr node) {
+    if (node == NULL) {
+	fprintf(output, "NULL\n");
+	return;
+    }
     switch (node->type) {
 	case XML_ELEMENT_NODE:
 	    fprintf(output, "-");
@@ -1307,7 +1380,7 @@ xmlShellPrintNode(xmlNodePtr node)
 
 /**
  * xmlShellPrintXPathResult:
- * list: a valid result generated by an xpath evaluation
+ * @list: a valid result generated by an xpath evaluation
  *
  * Prints result to stdout
  */
@@ -1373,7 +1446,10 @@ xmlShellList(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
              xmlNodePtr node2 ATTRIBUTE_UNUSED)
 {
     xmlNodePtr cur;
-
+    if (node == NULL) {
+	fprintf(stdout, "NULL\n");
+	return (0);
+    }
     if ((node->type == XML_DOCUMENT_NODE) ||
         (node->type == XML_HTML_DOCUMENT_NODE)) {
         cur = ((xmlDocPtr) node)->children;
@@ -1411,6 +1487,10 @@ xmlShellBase(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
              xmlNodePtr node2 ATTRIBUTE_UNUSED)
 {
     xmlChar *base;
+    if (node == NULL) {
+	fprintf(stdout, "NULL\n");
+	return (0);
+    }    
 
     base = xmlNodeGetBase(node->doc, node);
 
@@ -1423,6 +1503,27 @@ xmlShellBase(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
     return (0);
 }
 
+/**
+ * xmlShellSetBase:
+ * @ctxt:  the shell context
+ * @arg:  the new base
+ * @node:  a node
+ * @node2:  unused
+ *
+ * Implements the XML shell function "setbase"
+ * change the current XML base of the node
+ *
+ * Returns 0
+ */
+static int
+xmlShellSetBase(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
+             char *arg ATTRIBUTE_UNUSED, xmlNodePtr node,
+             xmlNodePtr node2 ATTRIBUTE_UNUSED)
+{
+    xmlNodeSetBase(node, (xmlChar*) arg);
+    return (0);
+}
+
 /**
  * xmlShellDir:
  * @ctxt:  the shell context
@@ -1440,6 +1541,10 @@ xmlShellDir(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
             char *arg ATTRIBUTE_UNUSED, xmlNodePtr node,
             xmlNodePtr node2 ATTRIBUTE_UNUSED)
 {
+    if (node == NULL) {
+	fprintf(stdout, "NULL\n");
+	return (0);
+    }    
     if ((node->type == XML_DOCUMENT_NODE) ||
         (node->type == XML_HTML_DOCUMENT_NODE)) {
         xmlDebugDumpDocumentHead(stdout, (xmlDocPtr) node);
@@ -1467,6 +1572,10 @@ int
 xmlShellCat(xmlShellCtxtPtr ctxt, char *arg ATTRIBUTE_UNUSED,
             xmlNodePtr node, xmlNodePtr node2 ATTRIBUTE_UNUSED)
 {
+    if (node == NULL) {
+	fprintf(stdout, "NULL\n");
+	return (0);
+    }    
     if (ctxt->doc->type == XML_HTML_DOCUMENT_NODE) {
 #ifdef LIBXML_HTML_ENABLED
         if (node->type == XML_HTML_DOCUMENT_NODE)
@@ -1792,7 +1901,7 @@ xmlShellDu(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
  * xmlShellPwd:
  * @ctxt:  the shell context
  * @buffer:  the output buffer
- * @tree:  a node 
+ * @node:  a node 
  * @node2:  unused
  *
  * Implements the XML shell function "pwd"
@@ -1940,6 +2049,7 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
             break;
 		if (!strcmp(command, "help")) {
 		  fprintf(stdout, "\tbase         display XML base of the node\n");
+		  fprintf(stdout, "\tsetbase URI  change the XML base of the node\n");
 		  fprintf(stdout, "\tbye          leave shell\n");
 		  fprintf(stdout, "\tcat [node]   display node or current node\n");
 		  fprintf(stdout, "\tcd [path]    change directory to path or to root\n");
@@ -1950,6 +2060,9 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 		  fprintf(stdout, "\tfree         display memory usage\n");
 		  fprintf(stdout, "\tload [name]  load a new document with name\n");
 		  fprintf(stdout, "\tls [path]    list contents of path or the current directory\n");
+#ifdef LIBXML_XPATH_ENABLED
+		  fprintf(stdout, "\txpath expr   evaluate the XPath expression in that context and print the result\n");
+#endif /* LIBXML_XPATH_ENABLED */
 		  fprintf(stdout, "\tpwd          display current working directory\n");
 		  fprintf(stdout, "\tquit         leave shell\n");
 		  fprintf(stdout, "\tsave [name]  save this document to name or the original name\n");
@@ -1981,6 +2094,20 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
             xmlShellDu(ctxt, NULL, ctxt->node, NULL);
         } else if (!strcmp(command, "base")) {
             xmlShellBase(ctxt, NULL, ctxt->node, NULL);
+#ifdef LIBXML_XPATH_ENABLED
+        } else if (!strcmp(command, "xpath")) {
+            if (arg[0] == 0) {
+		xmlGenericError(xmlGenericErrorContext,
+				"xpath: expression required\n");
+	    } else {
+                ctxt->pctxt->node = ctxt->node;
+                list = xmlXPathEval((xmlChar *) arg, ctxt->pctxt);
+		xmlXPathDebugDumpObject(stdout, list, 0);
+		xmlXPathFreeObject(list);
+	    }
+#endif /* LIBXML_XPATH_ENABLED */
+        } else if (!strcmp(command, "setbase")) {
+            xmlShellSetBase(ctxt, arg, ctxt->node, NULL);
         } else if ((!strcmp(command, "ls")) || (!strcmp(command, "dir"))) {
             int dir = (!strcmp(command, "dir"));
 
