@@ -42,6 +42,11 @@ typedef xmlParserInput *xmlParserInputPtr;
 typedef struct _xmlParserCtxt xmlParserCtxt;
 typedef xmlParserCtxt *xmlParserCtxtPtr;
 
+/**
+ * BASE_BUFFER_SIZE:
+ *
+ * default buffer size 4000
+ */
 #define BASE_BUFFER_SIZE 4000
 
 /**
@@ -507,7 +512,6 @@ struct _xmlDoc {
 /*
  * Variables.
  */
-LIBXML_DLL_IMPORT extern xmlNsPtr baseDTD;
 LIBXML_DLL_IMPORT extern int oldXMLWDcompatibility;/* maintain compatibility with old WD */
 LIBXML_DLL_IMPORT extern int xmlIndentTreeOutput;  /* try to indent the tree dumps */
 LIBXML_DLL_IMPORT extern xmlBufferAllocationScheme xmlBufferAllocScheme; /* alloc scheme to use */
@@ -787,11 +791,11 @@ int		xmlReconciliateNs	(xmlDocPtr doc,
  * Saving
  */
 void		xmlDocDumpFormatMemory	(xmlDocPtr cur,
-					 xmlChar**mem,
+					 xmlChar **mem,
 					 int *size,
 					 int format);
 void		xmlDocDumpMemory	(xmlDocPtr cur,
-					 xmlChar**mem,
+					 xmlChar **mem,
 					 int *size);
 void		xmlDocDumpMemoryEnc	(xmlDocPtr out_doc,
 					 xmlChar **doc_txt_ptr,
